"use strict";
/**
 * @file Fonctions qui gerent l'autorisation des utilisateurs
 * @author ofacklam
 * @memberof GraphQL
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const tools_1 = require("./tools");
const user_1 = require("../../ldap/export/user");
/*
 * There are 7 levels of authorization
 *  none : doesn't know the group / the user exists
 *  connectedOrOnplatal : knows the group exists, can use TOL
 *  viewer : can see the group
 *  member : part of the group
 *  speaker : allowed to speak for the group
 *  admin : admin of the group
 *  supervisor : allowed to take control of the group
 */
class AuthorizationModel {
    /**
     * @memberof GraphQL
     * @class AuthorizationModel
     * @summary Autorisation des utilisateurs.
     * @classdesc Cette classe contient les méthodes d'autorisation de l'utilisateur dans le 'context'.
     * @arg {string} uid - L'identifiant de l'utilisateur.
     */
    constructor(uid) {
        this.uid = uid;
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui crée une nouvelle instance de AuthorizationModel et charge les données
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @returns {Promise(AuthorizationModel)} Renvoie un nouveau AuthorizationModel avec cet utilisateur
     * @async
     * @static
     */
    static create(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            let model = new AuthorizationModel(uid);
            console.log("constructing AuthorizationModel...");
            if (model.isAuthenticated()) {
                console.log("model is authenticated. fetching data...");
                yield model.fetchData();
            }
            console.log("finished constructing AuthorizationModel");
            return model;
        });
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function fetchData
     * @summary Fonction qui recupere toutes les données pour populer les champs a partir de l'uid.
     * @async
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log("calling UT.peek from ldap connector (User Tool)...");
            let data = yield user_1.User.peek(this.uid);
            console.log("UT.peek returned with data:");
            console.log(data);
            this.viewerOf = yield tools_1.Tools.viewerOf(data);
            this.memberOf = yield tools_1.Tools.memberOf(data);
            this.speakerOf = yield tools_1.Tools.viewerOf(data);
            this.adminOf = yield tools_1.Tools.adminOf(data);
            this.supervisorOf = yield tools_1.Tools.supervisorOf(data);
        });
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function getUid
     * @summary Fonction qui renvoit l'identifiant de l'utilisateur
     * @return {string} Renvoie l'uid de l'utilisateur du 'context'
     */
    getUid() {
        return this.uid;
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function isConnectedOrOnplatal
     * @summary Fonction qui renvoit si l'utilisateur est connecté ou on-platal
     * @return {boolean} Renvoie true si l'utilisateur est connecté ou on-platal
     */
    isConnectedOrOnplatal() {
        return (this.uid != AuthorizationModel.PUBLICUSER);
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function isAuthenticated
     * @summary Fonction qui renvoit si l'utilisateur est authentifié
     * @return {boolean} Renvoie true si l'utilisateur est authentifié
     */
    isAuthenticated() {
        console.log("running model.isAuthenticated...");
        return (this.uid != AuthorizationModel.PUBLICUSER && this.uid != AuthorizationModel.ONPLATALUSER);
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function isViewer
     * @summary Fonction qui renvoit si l'utilisateur est viewer du groupe.
     * @arg {string} gid - Identifiant du groupe.
     * @return {boolean} Renvoie true si l'utilisateur est viewer du groupe.
     */
    isViewer(gid) {
        //ensure uid is valid !!!!!!!!
        if (this.isAuthenticated()) {
            let groups = this.viewerOf;
            return (groups.simpleGroups.has(gid) || groups.metaGroups.has(gid));
        }
        return false;
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function groupsViewer
     * @summary Fonction qui renvoit les groupes dont l'utilisateur est viewer.
     * @return {GroupCollection} Renvoie la collection de groupes dont l'utilisateur est viewer.
     */
    groupsViewer() {
        //ensure uid is valid !!!!!!!!
        if (this.isAuthenticated()) {
            return this.viewerOf;
        }
        return null;
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function isMember
     * @summary Fonction qui renvoit si l'utilisateur est membre du groupe.
     * @arg {string} gid - Identifiant du groupe.
     * @return {boolean} Renvoie true si l'utilisateur est membre du groupe.
     */
    isMember(gid) {
        //ensure uid is valid !!!!!!!!
        if (this.isAuthenticated()) {
            let groups = this.memberOf;
            return (groups.simpleGroups.has(gid) || groups.metaGroups.has(gid));
        }
        return false;
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function isMemberOr
     * @summary Fonction qui renvoit si l'utilisateur est membre d'au moins un de ces groupes.
     * @arg {GroupSet} groups - Ensemble de groupes.
     * @return {boolean} Renvoie true si l'utilisateur est membre d'un des groupes
     */
    isMemberOr(groups) {
        //ensure uid is valid !!!!!!!!
        if (this.isAuthenticated()) {
            let member = this.memberOf;
            for (let gid of groups) {
                if (member.simpleGroups.has(gid) || member.metaGroups.has(gid)) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function groupsMember
     * @summary Fonction qui renvoit les groupes dont l'utilisateur est member.
     * @return {GroupCollection} Renvoie la collection de groupes dont l'utilisateur est member.
     */
    groupsMember() {
        //ensure uid is valid !!!!!!!!
        if (this.isAuthenticated()) {
            return this.memberOf;
        }
        return null;
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function isSpeaker
     * @summary Fonction qui renvoit si l'utilisateur est speaker du groupe.
     * @arg {string} gid - Identifiant du groupe.
     * @return {boolean} Renvoie true si l'utilisateur est speaker du groupe.
     */
    isSpeaker(gid) {
        //ensure uid is valid !!!!!!!!
        if (this.isAuthenticated()) {
            let groups = this.speakerOf;
            return (groups.simpleGroups.has(gid) || groups.metaGroups.has(gid));
        }
        return false;
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function groupsSpeaker
     * @summary Fonction qui renvoit les groupes dont l'utilisateur est speaker.
     * @return {GroupCollection} Renvoie la collection de groupes dont l'utilisateur est speaker.
     */
    groupsSpeaker() {
        //ensure uid is valid !!!!!!!!
        if (this.isAuthenticated()) {
            return this.speakerOf;
        }
        return null;
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function isAdmin
     * @summary Fonction qui renvoit si l'utilisateur est admin du groupe.
     * @arg {string} gid - Identifiant du groupe.
     * @return {boolean} Renvoie true si l'utilisateur est admin du groupe.
     */
    isAdmin(gid) {
        //ensure uid is valid !!!!!!!!
        if (this.isAuthenticated()) {
            let groups = this.adminOf;
            let is_admin = groups.simpleGroups.has(gid) || groups.metaGroups.has(gid);
            //TODO : Vérifier s'il a pris les droits d'admin en étant supervisor
            let took_admin_rights = false;
            return (is_admin || took_admin_rights);
        }
        return false;
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function groupsAdmin
     * @summary Fonction qui renvoit les groupes dont l'utilisateur est admin.
     * @return {GroupCollection} Renvoie la collection de groupes dont l'utilisateur est admin.
     */
    groupsAdmin() {
        //ensure uid is valid !!!!!!!!
        if (this.isAuthenticated()) {
            return this.adminOf;
        }
        return null;
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function isSupervisor
     * @summary Fonction qui renvoit si l'utilisateur est supervisor du groupe.
     * @arg {string} gid - Identifiant du groupe.
     * @return {boolean} Renvoie true si l'utilisateur est supervisor du groupe.
     */
    isSupervisor(gid) {
        //ensure uid is valid !!!!!!!!
        if (this.isAuthenticated()) {
            let groups = this.supervisorOf;
            return (groups.simpleGroups.has(gid) || groups.metaGroups.has(gid));
        }
        return false;
    }
    /**
     * @memberof GraphQL.AuthorizationModel#
     * @function groupsSupervisor
     * @summary Fonction qui renvoit les groupes dont l'utilisateur est supervisor.
     * @return {GroupCollection} Renvoie la collection de groupes dont l'utilisateur est supervisor.
     */
    groupsSupervisor() {
        //ensure uid is valid !!!!!!!!
        if (this.isAuthenticated()) {
            return this.supervisorOf;
        }
        return null;
    }
}
AuthorizationModel.PUBLICUSER = "public_user";
AuthorizationModel.ONPLATALUSER = "public_onplatal_user";
exports.AuthorizationModel = AuthorizationModel;
//# sourceMappingURL=authorization.js.map