"use strict";
/**
 * @file Fonctions qui implémentent les requetes relatives aux groupes
 * @author ofacklam
 * @memberof GraphQL
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const groups_1 = require("../object_resolvers/groups");
const knex_router_1 = __importDefault(require("../../../db/knex_router"));
const tools_1 = require("./tools");
class GroupModel {
    /**
     * @memberof GraphQL
     * @class GroupModel
     * @summary Requetes relatives aux groupes.
     * @classdesc Cette classe contient les méthodes implémentant les requetes relatives aux groupes.
     * @arg {string} contextUser - L'identifiant de l'utilisateur du 'context'
     */
    constructor(contextUser) {
        this.contextUser = contextUser;
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function getGroup
     * @summary Fonction qui renvoie un groupe donné.
     * @arg {string} gid - Identifiant demandé.
     * @return {Promise(Group)} Renvoie le groupe dont l'identifiant est 'gid'
     * @async
     * @rights connectedOrOnplatal
     */
    getGroup(gid) {
        return __awaiter(this, void 0, void 0, function* () {
            let data = yield knex_router_1.default.select('type').from('groups').where('gid', gid);
            if (data.length > 0) {
                let type = data[0].type;
                switch (type) {
                    case 'simple':
                        return new groups_1.SimpleGroup(gid);
                    case 'meta':
                        return new groups_1.MetaGroup(gid);
                    default:
                        return null;
                }
            }
            return null;
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function getSimpleGroup
     * @summary Fonction qui renvoie un groupe simple donné.
     * @arg {string} gid - Identifiant demandé.
     * @return {Promise(SimpleGroup)} Renvoie le groupe dont l'identifiant est 'gid'
     * @async
     * @rights connectedOrOnplatal
     */
    getSimpleGroup(gid) {
        return __awaiter(this, void 0, void 0, function* () {
            return groups_1.SimpleGroup.tryCreate(gid);
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function getMetaGroup
     * @summary Fonction qui renvoie un méta-groupe donné.
     * @arg {string} gid - Identifiant demandé.
     * @return {Promise(MetaGroup)} Renvoie le groupe dont l'identifiant est 'gid'
     * @async
     * @rights connectedOrOnplatal
     */
    getMetaGroup(gid) {
        return __awaiter(this, void 0, void 0, function* () {
            return groups_1.MetaGroup.tryCreate(gid);
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function getAllGroupsByCollection
     * @summary Fonction qui renvoie tous les groupes donnés en argument.
     * @arg {GroupCollection} groups - Collection d'identifiants, supposés valides.
     * @return {Group[]} Renvoie le tableau de groupes correspondant
     * @rights connectedOrOnplatal
     */
    getAllGroupsByCollection(groups) {
        let res;
        for (let s of groups.simpleGroups) {
            res.push(new groups_1.SimpleGroup(s));
        }
        for (let m of groups.metaGroups) {
            res.push(new groups_1.MetaGroup(m));
        }
        return res;
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function getAllGroupsBySet
     * @summary Fonction qui renvoie tous les groupes donnés en argument.
     * @arg {GroupSet} groups - Ensemble d'identifiants, supposés valides.
     * @return {Promise(Group[])} Renvoie le tableau de groupes correspondant
     * @async
     * @rights connectedOrOnplatal
     */
    getAllGroupsBySet(groups) {
        return __awaiter(this, void 0, void 0, function* () {
            let res;
            for (let g of groups) {
                res.push(yield this.getGroup(g));
            }
            return res;
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function getAllSimpleGroups
     * @summary Fonction qui renvoie tous les groupes simples donnés en argument.
     * @arg {GroupSet} groups - Ensemble d'identifiants, supposés valides.
     * @return {SimpleGroup[]} Renvoie le tableau de groupes correspondant
     * @rights connectedOrOnplatal
     */
    getAllSimpleGroups(groups) {
        let res;
        for (let g of groups) {
            res.push(new groups_1.SimpleGroup(g));
        }
        return res;
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function likeGroup
     * @summary Fonction pour devenir sympathisant
     * @arg {string} gid - Identifiant du groupe.
     * @return {Promise(boolean)} Renvoie true si modification réussie.
     * @async
     * @rights viewer du groupe
     */
    likeGroup(gid) {
        return __awaiter(this, void 0, void 0, function* () {
            // Pour l'instant, ce n'est pas a implémenter...
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function unlikeGroup
     * @summary Fonction pour devenir non-sympathisant
     * @arg {string} gid - Identifiant du groupe.
     * @return {Promise(boolean)} Renvoie true si modification réussie.
     * @async
     * @rights viewer du groupe
     */
    unlikeGroup(gid) {
        return __awaiter(this, void 0, void 0, function* () {
            // Pour l'instant, ce n'est pas a implémenter...
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function userLeaveGroup
     * @summary Fonction pour quitter un groupe
     * @arg {string} gid - Identifiant du groupe.
     * @return {Promise(boolean)} Renvoie true si modification réussie.
     * @async
     * @rights member du groupe
     */
    userLeaveGroup(gid) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function writePostsSummary
     * @summary Fonction pour écrire le résumé des posts.
     * @arg {string} gid - Identifiant du groupe.
     * @arg {string} content - Contenu du résumé.
     * @return {Promise(boolean)} Renvoie true si modification réussie.
     * @async
     * @rights speaker du groupe
     */
    writePostsSummary(gid, content) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function createSubgroup
     * @summary Fonction pour créer un sous-groupe
     * @arg {createSubgroupArgs} args - Les arguments pour la création.
     * @return {Promise(Group)} Renvoie le groupe créé.
     * @async
     * @rights admin du groupe
     */
    createSubgroup(args) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
            //TODO : Vérifier que 'parent' est un groupe _simple_.
            // TODO : finish
            /*if (typeof args.fromGroup != 'string')
                throw "Illegal argument : parent_uid must be a non null string";
            if (typeof args.subName != 'string')
                throw "Illegal argument : name must be a non null string";
    
            let rasGID = await getAvailableGID(args.subGid);
    
            // TODO : appeller une fonction de LDAPUser pour y créer un groupe.
            await knex('simple_groups').insert({
                uid: rasGID,
                parent_uid: args.parent_uid,
                createdAt: knex.fn.now(),
                updatedAt: this.createdAt,
                name: args.name,
                website: args.website,
                description: args.description,
                school: args.school,
                type: "simple"
            });
    
            return getGroup(rasGID);*/
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function getAvailableGID
     * @summary Attribue un GID qui n'a pas encore été utilisé à un groupe
     * @desc Escape le string initialGID si necessaire (ramené à de l'ASCII sans espace), puis si le gid est deja pris rajoute un n a la fin et reteste
     * @arg {string} initialGID - Le gid initial du groupe, qu'il faut tester pour l'unicité.
     * @return {Promise(string)} Renvoie le gid unique pour ce groupe.
     * @rights authentified
     * remarque : n'importe qui peut tester si un groupe existe en demandant a créer un groupe avec ce nom la et en regardant si
     * son GID a été modifié. Je ne vois pas comment contourner ce problème, c'est donc une faille permanente (mineure) de sigma.
     * // ofacklam -> savoir qu'un groupe existe est autorisé pour toute personne connectée ou on-platal et il faut etre connecté pour
     * // utiliser cette fonction-ci donc il n'y a pas de probleme.
     */
    getAvailableGID(initialGID) {
        return __awaiter(this, void 0, void 0, function* () {
            let rasGID = tools_1.Tools.escapeID(initialGID);
            const res = knex_router_1.default.from('groups').where('gid', rasGID);
            if (res.length == 0) {
                return (rasGID);
            }
            else {
                return (this.getAvailableGID(rasGID + 'n'));
            }
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function makeAdmin
     * @summary Fonction pour rendre un membre admin
     * @arg {string} gid - Identifiant du groupe.
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @return {Promise(User)} Renvoie l'utilisateur promu.
     * @async
     * @rights admin du groupe
     */
    makeAdmin(gid, uid) {
        return __awaiter(this, void 0, void 0, function* () {
            //TODO : vérifier que l'utilisateur est bien déja membre !!
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function unmakeAdmin
     * @summary Fonction pour rendre un membre non-admin
     * @arg {string} gid - Identifiant du groupe.
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @return {Promise(User)} Renvoie l'utilisateur dé-promu.
     * @async
     * @rights admin du groupe
     */
    unmakeAdmin(gid, uid) {
        return __awaiter(this, void 0, void 0, function* () {
            //TODO : vérifier que l'utilisateur est bien déja membre !!
            //TODO : relacher les droits admin (des groupes enfants) pris en étant admin.
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function makeSpeaker
     * @summary Fonction pour rendre un membre speaker
     * @arg {string} gid - Identifiant du groupe.
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @return {Promise(User)} Renvoie l'utilisateur promu.
     * @async
     * @rights admin du groupe
     */
    makeSpeaker(gid, uid) {
        return __awaiter(this, void 0, void 0, function* () {
            //TODO : vérifier que l'utilisateur est bien déja membre !!
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function unmakeSpeaker
     * @summary Fonction pour rendre un membre non-speaker
     * @arg {string} gid - Identifiant du groupe.
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @return {Promise(User)} Renvoie l'utilisateur dé-promu.
     * @async
     * @rights admin du groupe
     */
    unmakeSpeaker(gid, uid) {
        return __awaiter(this, void 0, void 0, function* () {
            //TODO : vérifier que l'utilisateur est bien déja membre !!
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function editGroup
     * @summary Fonction pour modifier un groupe.
     * @arg {editGroupArgs} args - Les arguments de la modification.
     * @return {Promise(Group)} Renvoie le groupe modifié.
     * @async
     * @rights admin du groupe
     */
    editGroup(args) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function addVisibilityEdge
     * @summary Fonction pour ajouter une arete de visibilité.
     * @arg {string} forGroup - Le groupe a rendre visible.
     * @arg {string} visibleBy - Le groupe qui pourra voir.
     * @return {Promise(boolean)} Renvoie true si modification réussie.
     * @async
     * @rights admin du groupe 'forGroup'
     */
    addVisibilityEdge(forGroup, visibleBy) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function removeVisibilityEdge
     * @summary Fonction pour retirer une arete de visibilité.
     * @arg {string} forGroup - Le groupe a rendre invisible.
     * @arg {string} visibleBy - Le groupe qui ne pourra plus voir.
     * @return {Promise(boolean)} Renvoie true si modification réussie.
     * @async
     * @rights admin du groupe 'forGroup'
     */
    removeVisibilityEdge(forGroup, visibleBy) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function removeUser
     * @summary Fonction pour enlever un membre du groupe.
     * @arg {string} gid - Identifiant du groupe.
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @return {Promise(User)} Renvoie l'utilisateur.
     * @async
     * @rights admin du groupe
     */
    removeUser(gid, uid) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function takeAdminRights
     * @summary Fonction pour prendre les droits admin du groupe.
     * @arg {string} gid - Identifiant du groupe.
     * @arg {string} uid - Identifiant de l'utilisateur qui prend les droits.
     * @return {Promise(boolean)} Renvoie true si modification réussie.
     * @async
     * @rights supervisor du groupe
     */
    takeAdminRights(gid, uid) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
            /*await knex('taken_rights').insert({
                user_uid: user.uid,
                group_uid: groupUID,
                justification: justification
            });
            return true;*/
        });
    }
    /**
     * @memberof GraphQL.GroupModel#
     * @function releaseAdminRights
     * @summary Fonction pour relacher les droits admin du groupe.
     * @arg {string} gid - Identifiant du groupe.
     * @arg {string} uid - Identifiant de l'utilisateur qui relache les droits.
     * @return {Promise(boolean)} Renvoie true si modification réussie.
     * @async
     * @rights supervisor du groupe
     */
    releaseAdminRights(gid, uid) {
        return __awaiter(this, void 0, void 0, function* () {
            //TODO : relacher récursivement
            throw "Not implemented";
            /*return knex('taken_rights').del().where('user_uid', user.uid).where('group_uid', groupUID);*/
        });
    }
}
exports.GroupModel = GroupModel;
//# sourceMappingURL=groupModel.js.map